# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_options_common import ConnectionOptionsCommon
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsOAuth1Common(ConnectionOptionsCommon):
    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth 1.0 client identifier issued by the identity provider during application registration. This value identifies your Auth0 connection to the identity provider.
    """

    client_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth 1.0 client secret issued by the identity provider during application registration. Used to authenticate your Auth0 connection when signing requests and exchanging request tokens and verifiers for access tokens. May be null for public clients.
    """

    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class DailyStats(UniversalBaseModel):
    date: typing.Optional[str] = pydantic.Field(default=None)
    """
    Date these events occurred in ISO 8601 format.
    """

    logins: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of logins on this date.
    """

    signups: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of signups on this date.
    """

    leaked_passwords: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of breached-password detections on this date (subscription required).
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    Date and time this stats entry was last updated in ISO 8601 format.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    Approximate date and time the first event occurred in ISO 8601 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_totp_length_passwordless import ConnectionTotpLengthPasswordless

ConnectionTotpLengthSms = ConnectionTotpLengthPasswordless
"""
Number of digits in the verification code
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ClientAddonWsFed = typing.Dict[str, typing.Any]
"""
WS-Fed (WIF) addon indicator. Actual configuration is stored in `callback` and `client_aliases` properties on the client.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ClientAddonSalesforceApi(UniversalBaseModel):
    """
    Salesforce API addon configuration.
    """

    clientid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Consumer Key assigned by Salesforce to the Connected App.
    """

    principal: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the property in the user object that maps to a Salesforce username. e.g. `email`.
    """

    community_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="communityName"),
        pydantic.Field(alias="communityName", description="Community name."),
    ] = None
    """
    Community name.
    """

    community_url_section: typing.Optional[str] = pydantic.Field(default=None)
    """
    Community url section.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_events_subscribe() -> None:
    """Test subscribe endpoint with WireMock"""
    test_id = "events.subscribe.0"
    client = get_client(test_id)
    for _ in client.events.subscribe(
        from_="from",
        from_timestamp="from_timestamp",
        event_type=["connection.created"],
    ):
        pass
    verify_request_count(
        test_id,
        "GET",
        "/events",
        {"from": "from", "from_timestamp": "from_timestamp", "event_type": "connection.created"},
        1,
    )



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionSubjectTypesSupported = typing.List[str]
"""
A list of the Subject Identifier types that this OP supports. Valid types include pairwise and public
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventGroupMemberAddedCloudEventTypeEnum = typing.Union[typing.Literal["group.member.added"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.breached_password_detection_admin_notification_frequency_enum import (
    BreachedPasswordDetectionAdminNotificationFrequencyEnum,
)
from ...types.breached_password_detection_method_enum import BreachedPasswordDetectionMethodEnum
from ...types.breached_password_detection_shields_enum import BreachedPasswordDetectionShieldsEnum
from ...types.breached_password_detection_stage import BreachedPasswordDetectionStage
from ...types.get_breached_password_detection_settings_response_content import (
    GetBreachedPasswordDetectionSettingsResponseContent,
)
from ...types.update_breached_password_detection_settings_response_content import (
    UpdateBreachedPasswordDetectionSettingsResponseContent,
)
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBreachedPasswordDetectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBreachedPasswordDetectionSettingsResponseContent]:
        """
        Retrieve details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBreachedPasswordDetectionSettingsResponseContent]
            Breached password detection settings successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/breached-password-detection",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBreachedPasswordDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=GetBreachedPasswordDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]] = OMIT,
        admin_notification_frequency: typing.Optional[
            typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]
        ] = OMIT,
        method: typing.Optional[BreachedPasswordDetectionMethodEnum] = OMIT,
        stage: typing.Optional[BreachedPasswordDetectionStage] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBreachedPasswordDetectionSettingsResponseContent]:
        """
        Update details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not breached password detection is active.

        shields : typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]]
            Action to take when a breached password is detected during a login.
                  Possible values: <code>block</code>, <code>user_notification</code>, <code>admin_notification</code>.

        admin_notification_frequency : typing.Optional[typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]]
            When "admin_notification" is enabled, determines how often email notifications are sent.
                    Possible values: <code>immediately</code>, <code>daily</code>, <code>weekly</code>, <code>monthly</code>.

        method : typing.Optional[BreachedPasswordDetectionMethodEnum]

        stage : typing.Optional[BreachedPasswordDetectionStage]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBreachedPasswordDetectionSettingsResponseContent]
            Breached password detection settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/breached-password-detection",
            method="PATCH",
            json={
                "enabled": enabled,
                "shields": shields,
                "admin_notification_frequency": admin_notification_frequency,
                "method": method,
                "stage": convert_and_respect_annotation_metadata(
                    object_=stage, annotation=BreachedPasswordDetectionStage, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBreachedPasswordDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateBreachedPasswordDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBreachedPasswordDetectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBreachedPasswordDetectionSettingsResponseContent]:
        """
        Retrieve details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBreachedPasswordDetectionSettingsResponseContent]
            Breached password detection settings successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/breached-password-detection",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBreachedPasswordDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=GetBreachedPasswordDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]] = OMIT,
        admin_notification_frequency: typing.Optional[
            typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]
        ] = OMIT,
        method: typing.Optional[BreachedPasswordDetectionMethodEnum] = OMIT,
        stage: typing.Optional[BreachedPasswordDetectionStage] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBreachedPasswordDetectionSettingsResponseContent]:
        """
        Update details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not breached password detection is active.

        shields : typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]]
            Action to take when a breached password is detected during a login.
                  Possible values: <code>block</code>, <code>user_notification</code>, <code>admin_notification</code>.

        admin_notification_frequency : typing.Optional[typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]]
            When "admin_notification" is enabled, determines how often email notifications are sent.
                    Possible values: <code>immediately</code>, <code>daily</code>, <code>weekly</code>, <code>monthly</code>.

        method : typing.Optional[BreachedPasswordDetectionMethodEnum]

        stage : typing.Optional[BreachedPasswordDetectionStage]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBreachedPasswordDetectionSettingsResponseContent]
            Breached password detection settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/breached-password-detection",
            method="PATCH",
            json={
                "enabled": enabled,
                "shields": shields,
                "admin_notification_frequency": admin_notification_frequency,
                "method": method,
                "stage": convert_and_respect_annotation_metadata(
                    object_=stage, annotation=BreachedPasswordDetectionStage, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBreachedPasswordDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateBreachedPasswordDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ClientJwtConfigurationScopes = typing.Dict[str, typing.Any]
"""
Configuration related to id token claims for the client.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClientAddonEchoSign(UniversalBaseModel):
    """
    Adobe EchoSign SSO configuration.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    Your custom domain found in your EchoSign URL. e.g. `https://acme-org.echosign.com` would be `acme-org`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ClientComplianceLevelEnum = typing.Union[
    typing.Literal["none", "fapi1_adv_pkj_par", "fapi1_adv_mtls_par", "fapi2_sp_pkj_mtls", "fapi2_sp_mtls_mtls"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .organization_member_role import OrganizationMemberRole


class OrganizationMember(UniversalBaseModel):
    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of this user.
    """

    picture: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL to a picture for this user.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this user.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address of this user.
    """

    roles: typing.Optional[typing.List[OrganizationMemberRole]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionScopeAzureAd = typing.List[str]
"""
OAuth 2.0 scopes to request from Azure AD during authentication. Each scope represents a permission (e.g., 'User.Read', 'Group.Read.All'). Only applies with Microsoft Identity Platform v2.0. See Microsoft Graph permissions reference for available scopes.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.group import Group
from ...types.list_role_groups_response_content import ListRoleGroupsResponseContent
from .raw_client import AsyncRawGroupsClient, RawGroupsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGroupsClient
        """
        return self._raw_client

    def get(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Group, ListRoleGroupsResponseContent]:
        """
        Lists the groups to which the specified role is assigned.

        Parameters
        ----------
        id : str
            Unique identifier for the role (service-generated).

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Group, ListRoleGroupsResponseContent]
            Role's groups successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.roles.groups.get(
            id="id",
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.get(id, from_=from_, take=take, request_options=request_options)

    def create(
        self, id: str, *, groups: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Assign one or more groups to a specified role.

        Parameters
        ----------
        id : str
            Unique identifier for the role (service-generated).

        groups : typing.Sequence[str]
            Array of group IDs to assign to the role.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.roles.groups.create(
            id="id",
            groups=["groups"],
        )
        """
        _response = self._raw_client.create(id, groups=groups, request_options=request_options)
        return _response.data

    def delete(
        self, id: str, *, groups: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Unassign one or more groups from a specified role.

        Parameters
        ----------
        id : str
            Unique identifier for the role (service-generated).

        groups : typing.Sequence[str]
            Array of group IDs to remove from the role.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.roles.groups.delete(
            id="id",
            groups=["groups"],
        )
        """
        _response = self._raw_client.delete(id, groups=groups, request_options=request_options)
        return _response.data


class AsyncGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGroupsClient
        """
        return self._raw_client

    async def get(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Group, ListRoleGroupsResponseContent]:
        """
        Lists the groups to which the specified role is assigned.

        Parameters
        ----------
        id : str
            Unique identifier for the role (service-generated).

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Group, ListRoleGroupsResponseContent]
            Role's groups successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.roles.groups.get(
                id="id",
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.get(id, from_=from_, take=take, request_options=request_options)

    async def create(
        self, id: str, *, groups: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Assign one or more groups to a specified role.

        Parameters
        ----------
        id : str
            Unique identifier for the role (service-generated).

        groups : typing.Sequence[str]
            Array of group IDs to assign to the role.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.roles.groups.create(
                id="id",
                groups=["groups"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, groups=groups, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, groups: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Unassign one or more groups from a specified role.

        Parameters
        ----------
        id : str
            Unique identifier for the role (service-generated).

        groups : typing.Sequence[str]
            Array of group IDs to remove from the role.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.roles.groups.delete(
                id="id",
                groups=["groups"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, groups=groups, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

ConnectionUseCommonEndpointAzureAd = bool
"""
When enabled (true), uses the Azure AD common endpoint for multi-tenant authentication. Allows users from any Azure AD organization to sign in. Requires userid_attribute set to 'sub' (not 'oid'). Cannot be used with SCIM provisioning. Defaults to false.
"""



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.get_refresh_token_response_content import GetRefreshTokenResponseContent
from ..types.get_refresh_tokens_paginated_response_content import GetRefreshTokensPaginatedResponseContent
from ..types.refresh_token_metadata import RefreshTokenMetadata
from ..types.refresh_token_response_content import RefreshTokenResponseContent
from ..types.update_refresh_token_response_content import UpdateRefreshTokenResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRefreshTokensClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]:
        """
        Retrieve a paginated list of refresh tokens for a specific user, with optional filtering by client ID. Results are sorted by credential_id ascending.

        Parameters
        ----------
        user_id : str
            ID of the user whose refresh tokens to retrieve. Required.

        client_id : typing.Optional[str]
            Filter results by client ID. Only valid when user_id is provided.

        from_ : typing.Optional[str]
            An opaque cursor from which to start the selection (exclusive). Expires after 24 hours. Obtained from the next property of a previous response.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]
            The refresh tokens were retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "refresh-tokens",
            method="GET",
            params={
                "user_id": user_id,
                "client_id": client_id,
                "from": from_,
                "take": take,
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetRefreshTokensPaginatedResponseContent,
                    parse_obj_as(
                        type_=GetRefreshTokensPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.refresh_tokens
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    user_id=user_id,
                    client_id=client_id,
                    from_=_parsed_next,
                    take=take,
                    fields=fields,
                    include_fields=include_fields,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def revoke(
        self,
        *,
        ids: typing.Optional[typing.Sequence[str]] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        audience: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Revoke refresh tokens in bulk by ID list, user, user+client, or user+client+audience.

        Parameters
        ----------
        ids : typing.Optional[typing.Sequence[str]]
            Array of refresh token IDs to revoke. Limited to 100 at a time.

        user_id : typing.Optional[str]
            Revoke all refresh tokens for this user.

        client_id : typing.Optional[str]
            Revoke refresh tokens for this client. Must be paired with `user_id`; optionally narrowed further with `audience`.

        audience : typing.Optional[str]
            Resource server identifier (audience) to scope the revocation. Must be used with both `user_id` and `client_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "refresh-tokens/revoke",
            method="POST",
            json={
                "ids": ids,
                "user_id": user_id,
                "client_id": client_id,
                "audience": audience,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetRefreshTokenResponseContent]:
        """
        Retrieve refresh token information.

        Parameters
        ----------
        id : str
            ID refresh token to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetRefreshTokenResponseContent]
            The refresh token was retrieved
        """
        _response = self._client_wrapper.httpx_client.request(
            f"refresh-tokens/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetRefreshTokenResponseContent,
                    parse_obj_as(
                        type_=GetRefreshTokenResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"refresh-tokens/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        refresh_token_metadata: typing.Optional[RefreshTokenMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateRefreshTokenResponseContent]:
        """
        Update a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to update.

        refresh_token_metadata : typing.Optional[RefreshTokenMetadata]
            Metadata associated with the refresh token. Pass null or {} to remove all metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateRefreshTokenResponseContent]
            Refresh token successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"refresh-tokens/{encode_path_param(id)}",
            method="PATCH",
            json={
                "refresh_token_metadata": refresh_token_metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateRefreshTokenResponseContent,
                    parse_obj_as(
                        type_=UpdateRefreshTokenResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRefreshTokensClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]:
        """
        Retrieve a paginated list of refresh tokens for a specific user, with optional filtering by client ID. Results are sorted by credential_id ascending.

        Parameters
        ----------
        user_id : str
            ID of the user whose refresh tokens to retrieve. Required.

        client_id : typing.Optional[str]
            Filter results by client ID. Only valid when user_id is provided.

        from_ : typing.Optional[str]
            An opaque cursor from which to start the selection (exclusive). Expires after 24 hours. Obtained from the next property of a previous response.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]
            The refresh tokens were retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "refresh-tokens",
            method="GET",
            params={
                "user_id": user_id,
                "client_id": client_id,
                "from": from_,
                "take": take,
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetRefreshTokensPaginatedResponseContent,
                    parse_obj_as(
                        type_=GetRefreshTokensPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.refresh_tokens
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        user_id=user_id,
                        client_id=client_id,
                        from_=_parsed_next,
                        take=take,
                        fields=fields,
                        include_fields=include_fields,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def revoke(
        self,
        *,
        ids: typing.Optional[typing.Sequence[str]] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        audience: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Revoke refresh tokens in bulk by ID list, user, user+client, or user+client+audience.

        Parameters
        ----------
        ids : typing.Optional[typing.Sequence[str]]
            Array of refresh token IDs to revoke. Limited to 100 at a time.

        user_id : typing.Optional[str]
            Revoke all refresh tokens for this user.

        client_id : typing.Optional[str]
            Revoke refresh tokens for this client. Must be paired with `user_id`; optionally narrowed further with `audience`.

        audience : typing.Optional[str]
            Resource server identifier (audience) to scope the revocation. Must be used with both `user_id` and `client_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "refresh-tokens/revoke",
            method="POST",
            json={
                "ids": ids,
                "user_id": user_id,
                "client_id": client_id,
                "audience": audience,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetRefreshTokenResponseContent]:
        """
        Retrieve refresh token information.

        Parameters
        ----------
        id : str
            ID refresh token to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetRefreshTokenResponseContent]
            The refresh token was retrieved
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"refresh-tokens/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetRefreshTokenResponseContent,
                    parse_obj_as(
                        type_=GetRefreshTokenResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"refresh-tokens/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        refresh_token_metadata: typing.Optional[RefreshTokenMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateRefreshTokenResponseContent]:
        """
        Update a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to update.

        refresh_token_metadata : typing.Optional[RefreshTokenMetadata]
            Metadata associated with the refresh token. Pass null or {} to remove all metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateRefreshTokenResponseContent]
            Refresh token successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"refresh-tokens/{encode_path_param(id)}",
            method="PATCH",
            json={
                "refresh_token_metadata": refresh_token_metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateRefreshTokenResponseContent,
                    parse_obj_as(
                        type_=UpdateRefreshTokenResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.pagination import AsyncPager, SyncPager
from ....core.request_options import RequestOptions
from ....types.create_flows_vault_connection_request_content import CreateFlowsVaultConnectionRequestContent
from ....types.create_flows_vault_connection_response_content import CreateFlowsVaultConnectionResponseContent
from ....types.flows_vault_connection_summary import FlowsVaultConnectionSummary
from ....types.get_flows_vault_connection_response_content import GetFlowsVaultConnectionResponseContent
from ....types.list_flows_vault_connections_offset_paginated_response_content import (
    ListFlowsVaultConnectionsOffsetPaginatedResponseContent,
)
from ....types.update_flows_vault_connection_response_content import UpdateFlowsVaultConnectionResponseContent
from ....types.update_flows_vault_connection_setup import UpdateFlowsVaultConnectionSetup
from .raw_client import AsyncRawConnectionsClient, RawConnectionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConnectionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[FlowsVaultConnectionSummary, ListFlowsVaultConnectionsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[FlowsVaultConnectionSummary, ListFlowsVaultConnectionsOffsetPaginatedResponseContent]
            Vault connections successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.flows.vault.connections.list(
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def create(
        self,
        *,
        request: CreateFlowsVaultConnectionRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFlowsVaultConnectionResponseContent:
        """
        Parameters
        ----------
        request : CreateFlowsVaultConnectionRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFlowsVaultConnectionResponseContent
            Connection successfully created.

        Examples
        --------
        from auth0 import (
            Auth0,
            CreateFlowsVaultConnectionActivecampaignApiKey,
            FlowsVaultConnectioSetupApiKeyWithBaseUrl,
        )

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.vault.connections.create(
            request=CreateFlowsVaultConnectionActivecampaignApiKey(
                name="name",
                app_id="ACTIVECAMPAIGN",
                setup=FlowsVaultConnectioSetupApiKeyWithBaseUrl(
                    type="API_KEY",
                    api_key="api_key",
                    base_url="base_url",
                ),
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetFlowsVaultConnectionResponseContent:
        """
        Parameters
        ----------
        id : str
            Flows Vault connection ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFlowsVaultConnectionResponseContent
            Flow vault connection successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.vault.connections.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Vault connection id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.vault.connections.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        setup: typing.Optional[UpdateFlowsVaultConnectionSetup] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateFlowsVaultConnectionResponseContent:
        """
        Parameters
        ----------
        id : str
            Flows Vault connection ID

        name : typing.Optional[str]
            Flows Vault Connection name.

        setup : typing.Optional[UpdateFlowsVaultConnectionSetup]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateFlowsVaultConnectionResponseContent
            Flow vault connection successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.vault.connections.update(
            id="id",
        )
        """
        _response = self._raw_client.update(id, name=name, setup=setup, request_options=request_options)
        return _response.data


class AsyncConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConnectionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[FlowsVaultConnectionSummary, ListFlowsVaultConnectionsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[FlowsVaultConnectionSummary, ListFlowsVaultConnectionsOffsetPaginatedResponseContent]
            Vault connections successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.flows.vault.connections.list(
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def create(
        self,
        *,
        request: CreateFlowsVaultConnectionRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFlowsVaultConnectionResponseContent:
        """
        Parameters
        ----------
        request : CreateFlowsVaultConnectionRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFlowsVaultConnectionResponseContent
            Connection successfully created.

        Examples
        --------
        import asyncio

        from auth0 import (
            AsyncAuth0,
            CreateFlowsVaultConnectionActivecampaignApiKey,
            FlowsVaultConnectioSetupApiKeyWithBaseUrl,
        )

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.vault.connections.create(
                request=CreateFlowsVaultConnectionActivecampaignApiKey(
                    name="name",
                    app_id="ACTIVECAMPAIGN",
                    setup=FlowsVaultConnectioSetupApiKeyWithBaseUrl(
                        type="API_KEY",
                        api_key="api_key",
                        base_url="base_url",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetFlowsVaultConnectionResponseContent:
        """
        Parameters
        ----------
        id : str
            Flows Vault connection ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFlowsVaultConnectionResponseContent
            Flow vault connection successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.vault.connections.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Vault connection id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.vault.connections.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        setup: typing.Optional[UpdateFlowsVaultConnectionSetup] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateFlowsVaultConnectionResponseContent:
        """
        Parameters
        ----------
        id : str
            Flows Vault connection ID

        name : typing.Optional[str]
            Flows Vault Connection name.

        setup : typing.Optional[UpdateFlowsVaultConnectionSetup]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateFlowsVaultConnectionResponseContent
            Flow vault connection successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.vault.connections.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, name=name, setup=setup, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_log_stream_request_content import CreateLogStreamRequestContent
from ..types.create_log_stream_response_content import CreateLogStreamResponseContent
from ..types.get_log_stream_response_content import GetLogStreamResponseContent
from ..types.log_stream_filter import LogStreamFilter
from ..types.log_stream_pii_config import LogStreamPiiConfig
from ..types.log_stream_response_schema import LogStreamResponseSchema
from ..types.log_stream_sink_patch import LogStreamSinkPatch
from ..types.log_stream_status_enum import LogStreamStatusEnum
from ..types.update_log_stream_response_content import UpdateLogStreamResponseContent
from .raw_client import AsyncRawLogStreamsClient, RawLogStreamsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LogStreamsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLogStreamsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLogStreamsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLogStreamsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[LogStreamResponseSchema]:
        """
        Retrieve details on [log streams](https://auth0.com/docs/logs/streams).

        **Sample Response**

        ```json
        [{
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }, {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "eventgrid",
          "status": "active|paused|suspended",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }]
        ```

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[LogStreamResponseSchema]
            Returning log streams

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.log_streams.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def create(
        self, *, request: CreateLogStreamRequestContent, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateLogStreamResponseContent:
        """
        Create a log stream.

        **Log Stream Types**

        The `type` of log stream being created determines the properties required in the `sink` payload.

        **HTTP Stream**

        For an `http` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "http",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Amazon EventBridge Stream**

        For an `eventbridge` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventbridge",
          "sink": {
            "awsRegion": "string",
            "awsAccountId": "string"
          }
        }
        ```

        The response will include an additional field `awsPartnerEventSource` in the `sink`:

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **Azure Event Grid Stream**

        For an `Azure Event Grid` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventgrid",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        }
        ```

        **Datadog Stream**

        For a `Datadog` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "datadog",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Splunk Stream**

        For a `Splunk` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "splunk",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Stream**

        For a `Sumo Logic` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "sumo",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        request : CreateLogStreamRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLogStreamResponseContent
            Log stream created

        Examples
        --------
        from auth0 import Auth0, CreateLogStreamHttpRequestBody, LogStreamHttpSink

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.log_streams.create(
            request=CreateLogStreamHttpRequestBody(
                type="http",
                sink=LogStreamHttpSink(
                    http_endpoint="httpEndpoint",
                ),
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetLogStreamResponseContent:
        """
        Retrieve a log stream configuration and status.

        **Sample responses**

        **Amazon EventBridge Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **HTTP Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        }
        ```

        **Datadog Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Mixpanel**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "mixpanel",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "mixpanel",
          "status": "active",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Segment**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "segment",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "segment",
          "status": "active",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Splunk Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Status**

        The `status` of a log stream maybe any of the following:

        1. `active` - Stream is currently enabled.
        2. `paused` - Stream is currently user disabled and will not attempt log delivery.
        3. `suspended` - Stream is currently disabled because of errors and will not attempt log delivery.

        Parameters
        ----------
        id : str
            The id of the log stream to get

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLogStreamResponseContent
            Returning log stream.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.log_streams.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a log stream.

        Parameters
        ----------
        id : str
            The id of the log stream to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.log_streams.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[LogStreamStatusEnum] = OMIT,
        is_priority: typing.Optional[bool] = OMIT,
        filters: typing.Optional[typing.Sequence[LogStreamFilter]] = OMIT,
        pii_config: typing.Optional[LogStreamPiiConfig] = OMIT,
        sink: typing.Optional[LogStreamSinkPatch] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLogStreamResponseContent:
        """
        Update a log stream.

        **Examples of how to use the PATCH endpoint.**

        The following fields may be updated in a PATCH operation:

        - name
        - status
        - sink

        Note: For log streams of type `eventbridge` and `eventgrid`, updating the `sink` is not permitted.

        **Update the status of a log stream**

        ```json
        {
          "status": "active|paused"
        }
        ```

        **Update the name of a log stream**

        ```json
        {
          "name": "string"
        }
        ```

        **Update the sink properties of a stream of type `http`**

        ```json
        {
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONARRAY|JSONLINES",
            "httpAuthorization": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `datadog`**

        ```json
        {
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `splunk`**

        ```json
        {
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Update the sink properties of a stream of type `sumo`**

        ```json
        {
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        id : str
            The id of the log stream to get

        name : typing.Optional[str]
            log stream name

        status : typing.Optional[LogStreamStatusEnum]

        is_priority : typing.Optional[bool]
            True for priority log streams, false for non-priority

        filters : typing.Optional[typing.Sequence[LogStreamFilter]]
            Only logs events matching these filters will be delivered by the stream. If omitted or empty, all events will be delivered.

        pii_config : typing.Optional[LogStreamPiiConfig]

        sink : typing.Optional[LogStreamSinkPatch]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLogStreamResponseContent
            Log stream updated

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.log_streams.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            status=status,
            is_priority=is_priority,
            filters=filters,
            pii_config=pii_config,
            sink=sink,
            request_options=request_options,
        )
        return _response.data


class AsyncLogStreamsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLogStreamsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLogStreamsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLogStreamsClient
        """
        return self._raw_client

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[LogStreamResponseSchema]:
        """
        Retrieve details on [log streams](https://auth0.com/docs/logs/streams).

        **Sample Response**

        ```json
        [{
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }, {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "eventgrid",
          "status": "active|paused|suspended",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }]
        ```

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[LogStreamResponseSchema]
            Returning log streams

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.log_streams.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def create(
        self, *, request: CreateLogStreamRequestContent, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateLogStreamResponseContent:
        """
        Create a log stream.

        **Log Stream Types**

        The `type` of log stream being created determines the properties required in the `sink` payload.

        **HTTP Stream**

        For an `http` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "http",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Amazon EventBridge Stream**

        For an `eventbridge` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventbridge",
          "sink": {
            "awsRegion": "string",
            "awsAccountId": "string"
          }
        }
        ```

        The response will include an additional field `awsPartnerEventSource` in the `sink`:

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **Azure Event Grid Stream**

        For an `Azure Event Grid` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventgrid",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        }
        ```

        **Datadog Stream**

        For a `Datadog` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "datadog",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Splunk Stream**

        For a `Splunk` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "splunk",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Stream**

        For a `Sumo Logic` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "sumo",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        request : CreateLogStreamRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLogStreamResponseContent
            Log stream created

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, CreateLogStreamHttpRequestBody, LogStreamHttpSink

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.log_streams.create(
                request=CreateLogStreamHttpRequestBody(
                    type="http",
                    sink=LogStreamHttpSink(
                        http_endpoint="httpEndpoint",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLogStreamResponseContent:
        """
        Retrieve a log stream configuration and status.

        **Sample responses**

        **Amazon EventBridge Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **HTTP Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        }
        ```

        **Datadog Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Mixpanel**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "mixpanel",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "mixpanel",
          "status": "active",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Segment**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "segment",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "segment",
          "status": "active",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Splunk Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Status**

        The `status` of a log stream maybe any of the following:

        1. `active` - Stream is currently enabled.
        2. `paused` - Stream is currently user disabled and will not attempt log delivery.
        3. `suspended` - Stream is currently disabled because of errors and will not attempt log delivery.

        Parameters
        ----------
        id : str
            The id of the log stream to get

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLogStreamResponseContent
            Returning log stream.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.log_streams.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a log stream.

        Parameters
        ----------
        id : str
            The id of the log stream to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.log_streams.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[LogStreamStatusEnum] = OMIT,
        is_priority: typing.Optional[bool] = OMIT,
        filters: typing.Optional[typing.Sequence[LogStreamFilter]] = OMIT,
        pii_config: typing.Optional[LogStreamPiiConfig] = OMIT,
        sink: typing.Optional[LogStreamSinkPatch] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLogStreamResponseContent:
        """
        Update a log stream.

        **Examples of how to use the PATCH endpoint.**

        The following fields may be updated in a PATCH operation:

        - name
        - status
        - sink

        Note: For log streams of type `eventbridge` and `eventgrid`, updating the `sink` is not permitted.

        **Update the status of a log stream**

        ```json
        {
          "status": "active|paused"
        }
        ```

        **Update the name of a log stream**

        ```json
        {
          "name": "string"
        }
        ```

        **Update the sink properties of a stream of type `http`**

        ```json
        {
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONARRAY|JSONLINES",
            "httpAuthorization": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `datadog`**

        ```json
        {
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `splunk`**

        ```json
        {
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Update the sink properties of a stream of type `sumo`**

        ```json
        {
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        id : str
            The id of the log stream to get

        name : typing.Optional[str]
            log stream name

        status : typing.Optional[LogStreamStatusEnum]

        is_priority : typing.Optional[bool]
            True for priority log streams, false for non-priority

        filters : typing.Optional[typing.Sequence[LogStreamFilter]]
            Only logs events matching these filters will be delivered by the stream. If omitted or empty, all events will be delivered.

        pii_config : typing.Optional[LogStreamPiiConfig]

        sink : typing.Optional[LogStreamSinkPatch]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLogStreamResponseContent
            Log stream updated

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.log_streams.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            status=status,
            is_priority=is_priority,
            filters=filters,
            pii_config=pii_config,
            sink=sink,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connections_metadata import ConnectionsMetadata
from .self_service_profile_sso_ticket_connection_options import SelfServiceProfileSsoTicketConnectionOptions


class SelfServiceProfileSsoTicketConnectionConfig(UniversalBaseModel):
    """
    If provided, this will create a new connection for the Self-Service Enterprise Configuration flow with the given configuration
    """

    name: str = pydantic.Field()
    """
    The name of the connection that will be created as a part of the Self-Service Enterprise Configuration flow.
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Connection name used in the new universal login experience
    """

    is_domain_connection: typing.Optional[bool] = pydantic.Field(default=None)
    """
    <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)
    """

    show_as_button: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)
    """

    metadata: typing.Optional[ConnectionsMetadata] = None
    options: typing.Optional[SelfServiceProfileSsoTicketConnectionOptions] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_users_enrollments_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "users.enrollments.get.0"
    client = get_client(test_id)
    client.users.enrollments.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/users/id/enrollments", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_whatsapp_send_message_params_payload_object import FlowActionWhatsappSendMessageParamsPayloadObject

FlowActionWhatsappSendMessageParamsPayload = typing.Union[FlowActionWhatsappSendMessageParamsPayloadObject, str]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .flow_action_flow_boolean_condition_action import FlowActionFlowBooleanConditionAction
from .flow_action_flow_boolean_condition_type import FlowActionFlowBooleanConditionType


class FlowActionFlowBooleanCondition(UniversalBaseModel):
    id: str
    alias: typing.Optional[str] = None
    type: FlowActionFlowBooleanConditionType
    action: FlowActionFlowBooleanConditionAction
    allow_failure: typing.Optional[bool] = None
    mask_output: typing.Optional[bool] = None
    params: "FlowActionFlowBooleanConditionParams"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .flow_action import FlowAction  # noqa: E402, I001
from .flow_action_flow import FlowActionFlow  # noqa: E402, I001
from .flow_action_flow_boolean_condition_params import FlowActionFlowBooleanConditionParams  # noqa: E402, I001

update_forward_refs(
    FlowActionFlowBooleanCondition,
    FlowAction=FlowAction,
    FlowActionFlow=FlowActionFlow,
    FlowActionFlowBooleanConditionParams=FlowActionFlowBooleanConditionParams,
)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.create_guardian_enrollment_ticket_response_content import CreateGuardianEnrollmentTicketResponseContent
from ...types.get_guardian_enrollment_response_content import GetGuardianEnrollmentResponseContent
from ...types.guardian_enrollment_factor_enum import GuardianEnrollmentFactorEnum
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawEnrollmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_ticket(
        self,
        *,
        user_id: str,
        email: typing.Optional[str] = OMIT,
        send_mail: typing.Optional[bool] = OMIT,
        email_locale: typing.Optional[str] = OMIT,
        factor: typing.Optional[GuardianEnrollmentFactorEnum] = OMIT,
        allow_multiple_enrollments: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateGuardianEnrollmentTicketResponseContent]:
        """
        Create a [multi-factor authentication (MFA) enrollment ticket](https://auth0.com/docs/secure/multi-factor-authentication/auth0-guardian/create-custom-enrollment-tickets), and optionally send an email with the created ticket to a given user. Enrollment tickets can specify which factor users must enroll with or allow existing MFA users to enroll in additional factors.

        Parameters
        ----------
        user_id : str
            user_id for the enrollment ticket

        email : typing.Optional[str]
            alternate email to which the enrollment email will be sent. Optional - by default, the email will be sent to the user's default address

        send_mail : typing.Optional[bool]
            Send an email to the user to start the enrollment

        email_locale : typing.Optional[str]
            Optional. Specify the locale of the enrollment email. Used with send_email.

        factor : typing.Optional[GuardianEnrollmentFactorEnum]

        allow_multiple_enrollments : typing.Optional[bool]
            Optional. Allows a user who has previously enrolled in MFA to enroll with additional factors.<br />Note: Parameter can only be used with Universal Login; it cannot be used with Classic Login or custom MFA pages.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateGuardianEnrollmentTicketResponseContent]
            Enrollment ticket successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/enrollments/ticket",
            method="POST",
            json={
                "user_id": user_id,
                "email": email,
                "send_mail": send_mail,
                "email_locale": email_locale,
                "factor": factor,
                "allow_multiple_enrollments": allow_multiple_enrollments,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateGuardianEnrollmentTicketResponseContent,
                    parse_obj_as(
                        type_=CreateGuardianEnrollmentTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGuardianEnrollmentResponseContent]:
        """
        Retrieve details, such as status and type, for a specific multi-factor authentication enrollment registered to a user account.

        Parameters
        ----------
        id : str
            ID of the enrollment to be retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGuardianEnrollmentResponseContent]
            Enrollment successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"guardian/enrollments/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianEnrollmentResponseContent,
                    parse_obj_as(
                        type_=GetGuardianEnrollmentResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Remove a specific multi-factor authentication (MFA) enrollment from a user's account. This allows the user to re-enroll with MFA. For more information, review [Reset User Multi-Factor Authentication and Recovery Codes](https://auth0.com/docs/secure/multi-factor-authentication/reset-user-mfa).

        Parameters
        ----------
        id : str
            ID of the enrollment to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"guardian/enrollments/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawEnrollmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_ticket(
        self,
        *,
        user_id: str,
        email: typing.Optional[str] = OMIT,
        send_mail: typing.Optional[bool] = OMIT,
        email_locale: typing.Optional[str] = OMIT,
        factor: typing.Optional[GuardianEnrollmentFactorEnum] = OMIT,
        allow_multiple_enrollments: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateGuardianEnrollmentTicketResponseContent]:
        """
        Create a [multi-factor authentication (MFA) enrollment ticket](https://auth0.com/docs/secure/multi-factor-authentication/auth0-guardian/create-custom-enrollment-tickets), and optionally send an email with the created ticket to a given user. Enrollment tickets can specify which factor users must enroll with or allow existing MFA users to enroll in additional factors.

        Parameters
        ----------
        user_id : str
            user_id for the enrollment ticket

        email : typing.Optional[str]
            alternate email to which the enrollment email will be sent. Optional - by default, the email will be sent to the user's default address

        send_mail : typing.Optional[bool]
            Send an email to the user to start the enrollment

        email_locale : typing.Optional[str]
            Optional. Specify the locale of the enrollment email. Used with send_email.

        factor : typing.Optional[GuardianEnrollmentFactorEnum]

        allow_multiple_enrollments : typing.Optional[bool]
            Optional. Allows a user who has previously enrolled in MFA to enroll with additional factors.<br />Note: Parameter can only be used with Universal Login; it cannot be used with Classic Login or custom MFA pages.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateGuardianEnrollmentTicketResponseContent]
            Enrollment ticket successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/enrollments/ticket",
            method="POST",
            json={
                "user_id": user_id,
                "email": email,
                "send_mail": send_mail,
                "email_locale": email_locale,
                "factor": factor,
                "allow_multiple_enrollments": allow_multiple_enrollments,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateGuardianEnrollmentTicketResponseContent,
                    parse_obj_as(
                        type_=CreateGuardianEnrollmentTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGuardianEnrollmentResponseContent]:
        """
        Retrieve details, such as status and type, for a specific multi-factor authentication enrollment registered to a user account.

        Parameters
        ----------
        id : str
            ID of the enrollment to be retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGuardianEnrollmentResponseContent]
            Enrollment successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"guardian/enrollments/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianEnrollmentResponseContent,
                    parse_obj_as(
                        type_=GetGuardianEnrollmentResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Remove a specific multi-factor authentication (MFA) enrollment from a user's account. This allows the user to re-enroll with MFA. For more information, review [Reset User Multi-Factor Authentication and Recovery Codes](https://auth0.com/docs/secure/multi-factor-authentication/reset-user-mfa).

        Parameters
        ----------
        id : str
            ID of the enrollment to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"guardian/enrollments/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.get_refresh_token_response_content import GetRefreshTokenResponseContent
from ..types.get_refresh_tokens_paginated_response_content import GetRefreshTokensPaginatedResponseContent
from ..types.refresh_token_metadata import RefreshTokenMetadata
from ..types.refresh_token_response_content import RefreshTokenResponseContent
from ..types.update_refresh_token_response_content import UpdateRefreshTokenResponseContent
from .raw_client import AsyncRawRefreshTokensClient, RawRefreshTokensClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RefreshTokensClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRefreshTokensClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRefreshTokensClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRefreshTokensClient
        """
        return self._raw_client

    def list(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]:
        """
        Retrieve a paginated list of refresh tokens for a specific user, with optional filtering by client ID. Results are sorted by credential_id ascending.

        Parameters
        ----------
        user_id : str
            ID of the user whose refresh tokens to retrieve. Required.

        client_id : typing.Optional[str]
            Filter results by client ID. Only valid when user_id is provided.

        from_ : typing.Optional[str]
            An opaque cursor from which to start the selection (exclusive). Expires after 24 hours. Obtained from the next property of a previous response.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]
            The refresh tokens were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.refresh_tokens.list(
            user_id="user_id",
            client_id="client_id",
            from_="from",
            take=1,
            fields="fields",
            include_fields=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            user_id=user_id,
            client_id=client_id,
            from_=from_,
            take=take,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )

    def revoke(
        self,
        *,
        ids: typing.Optional[typing.Sequence[str]] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        audience: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Revoke refresh tokens in bulk by ID list, user, user+client, or user+client+audience.

        Parameters
        ----------
        ids : typing.Optional[typing.Sequence[str]]
            Array of refresh token IDs to revoke. Limited to 100 at a time.

        user_id : typing.Optional[str]
            Revoke all refresh tokens for this user.

        client_id : typing.Optional[str]
            Revoke refresh tokens for this client. Must be paired with `user_id`; optionally narrowed further with `audience`.

        audience : typing.Optional[str]
            Resource server identifier (audience) to scope the revocation. Must be used with both `user_id` and `client_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.refresh_tokens.revoke()
        """
        _response = self._raw_client.revoke(
            ids=ids, user_id=user_id, client_id=client_id, audience=audience, request_options=request_options
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetRefreshTokenResponseContent:
        """
        Retrieve refresh token information.

        Parameters
        ----------
        id : str
            ID refresh token to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetRefreshTokenResponseContent
            The refresh token was retrieved

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.refresh_tokens.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.refresh_tokens.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        refresh_token_metadata: typing.Optional[RefreshTokenMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateRefreshTokenResponseContent:
        """
        Update a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to update.

        refresh_token_metadata : typing.Optional[RefreshTokenMetadata]
            Metadata associated with the refresh token. Pass null or {} to remove all metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateRefreshTokenResponseContent
            Refresh token successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.refresh_tokens.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id, refresh_token_metadata=refresh_token_metadata, request_options=request_options
        )
        return _response.data


class AsyncRefreshTokensClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRefreshTokensClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRefreshTokensClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRefreshTokensClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]:
        """
        Retrieve a paginated list of refresh tokens for a specific user, with optional filtering by client ID. Results are sorted by credential_id ascending.

        Parameters
        ----------
        user_id : str
            ID of the user whose refresh tokens to retrieve. Required.

        client_id : typing.Optional[str]
            Filter results by client ID. Only valid when user_id is provided.

        from_ : typing.Optional[str]
            An opaque cursor from which to start the selection (exclusive). Expires after 24 hours. Obtained from the next property of a previous response.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RefreshTokenResponseContent, GetRefreshTokensPaginatedResponseContent]
            The refresh tokens were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.refresh_tokens.list(
                user_id="user_id",
                client_id="client_id",
                from_="from",
                take=1,
                fields="fields",
                include_fields=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            user_id=user_id,
            client_id=client_id,
            from_=from_,
            take=take,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )

    async def revoke(
        self,
        *,
        ids: typing.Optional[typing.Sequence[str]] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        audience: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Revoke refresh tokens in bulk by ID list, user, user+client, or user+client+audience.

        Parameters
        ----------
        ids : typing.Optional[typing.Sequence[str]]
            Array of refresh token IDs to revoke. Limited to 100 at a time.

        user_id : typing.Optional[str]
            Revoke all refresh tokens for this user.

        client_id : typing.Optional[str]
            Revoke refresh tokens for this client. Must be paired with `user_id`; optionally narrowed further with `audience`.

        audience : typing.Optional[str]
            Resource server identifier (audience) to scope the revocation. Must be used with both `user_id` and `client_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refresh_tokens.revoke()


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke(
            ids=ids, user_id=user_id, client_id=client_id, audience=audience, request_options=request_options
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetRefreshTokenResponseContent:
        """
        Retrieve refresh token information.

        Parameters
        ----------
        id : str
            ID refresh token to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetRefreshTokenResponseContent
            The refresh token was retrieved

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refresh_tokens.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refresh_tokens.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        refresh_token_metadata: typing.Optional[RefreshTokenMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateRefreshTokenResponseContent:
        """
        Update a refresh token by its ID.

        Parameters
        ----------
        id : str
            ID of the refresh token to update.

        refresh_token_metadata : typing.Optional[RefreshTokenMetadata]
            Metadata associated with the refresh token. Pass null or {} to remove all metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateRefreshTokenResponseContent
            Refresh token successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refresh_tokens.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, refresh_token_metadata=refresh_token_metadata, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionCreatedObject2OptionsDigestAlgorithmEnum = typing.Union[
    typing.Literal["sha1", "sha256"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_telegram_token import CreateFlowsVaultConnectionTelegramToken
from .create_flows_vault_connection_telegram_uninitialized import CreateFlowsVaultConnectionTelegramUninitialized

CreateFlowsVaultConnectionTelegram = typing.Union[
    CreateFlowsVaultConnectionTelegramToken, CreateFlowsVaultConnectionTelegramUninitialized
]



# This file was auto-generated by Fern from our API Definition.

from .connection_provider_enum_sms import ConnectionProviderEnumSms

ConnectionProviderSms = ConnectionProviderEnumSms
"""
SMS provider. Set to 'sms_gateway' to use a custom SMS gateway instead of Twilio.
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawGuardianClient, RawGuardianClient

if typing.TYPE_CHECKING:
    from .enrollments.client import AsyncEnrollmentsClient, EnrollmentsClient
    from .factors.client import AsyncFactorsClient, FactorsClient
    from .policies.client import AsyncPoliciesClient, PoliciesClient


class GuardianClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGuardianClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._enrollments: typing.Optional[EnrollmentsClient] = None
        self._factors: typing.Optional[FactorsClient] = None
        self._policies: typing.Optional[PoliciesClient] = None

    @property
    def with_raw_response(self) -> RawGuardianClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGuardianClient
        """
        return self._raw_client

    @property
    def enrollments(self):
        if self._enrollments is None:
            from .enrollments.client import EnrollmentsClient  # noqa: E402

            self._enrollments = EnrollmentsClient(client_wrapper=self._client_wrapper)
        return self._enrollments

    @property
    def factors(self):
        if self._factors is None:
            from .factors.client import FactorsClient  # noqa: E402

            self._factors = FactorsClient(client_wrapper=self._client_wrapper)
        return self._factors

    @property
    def policies(self):
        if self._policies is None:
            from .policies.client import PoliciesClient  # noqa: E402

            self._policies = PoliciesClient(client_wrapper=self._client_wrapper)
        return self._policies


class AsyncGuardianClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGuardianClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._enrollments: typing.Optional[AsyncEnrollmentsClient] = None
        self._factors: typing.Optional[AsyncFactorsClient] = None
        self._policies: typing.Optional[AsyncPoliciesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawGuardianClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGuardianClient
        """
        return self._raw_client

    @property
    def enrollments(self):
        if self._enrollments is None:
            from .enrollments.client import AsyncEnrollmentsClient  # noqa: E402

            self._enrollments = AsyncEnrollmentsClient(client_wrapper=self._client_wrapper)
        return self._enrollments

    @property
    def factors(self):
        if self._factors is None:
            from .factors.client import AsyncFactorsClient  # noqa: E402

            self._factors = AsyncFactorsClient(client_wrapper=self._client_wrapper)
        return self._factors

    @property
    def policies(self):
        if self._policies is None:
            from .policies.client import AsyncPoliciesClient  # noqa: E402

            self._policies = AsyncPoliciesClient(client_wrapper=self._client_wrapper)
        return self._policies



# This file was auto-generated by Fern from our API Definition.

from .flow_action_mailchimp_upsert_member import FlowActionMailchimpUpsertMember

FlowActionMailchimp = FlowActionMailchimpUpsertMember



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_slack_oauth_code import CreateFlowsVaultConnectionSlackOauthCode
from .create_flows_vault_connection_slack_uninitialized import CreateFlowsVaultConnectionSlackUninitialized
from .create_flows_vault_connection_slack_webhook import CreateFlowsVaultConnectionSlackWebhook

CreateFlowsVaultConnectionSlack = typing.Union[
    CreateFlowsVaultConnectionSlackWebhook,
    CreateFlowsVaultConnectionSlackOauthCode,
    CreateFlowsVaultConnectionSlackUninitialized,
]
